@@ -14,6 +14,8 @@ has recursion_count => 0;
 has iteration_count => 0;
 
 has debug => ();
+has debug_indent => ();
+has debug_color => ();
 has recursion_limit => ();
 has recursion_warn_limit => ();
 has iteration_limit => ();
@@ -24,9 +26,17 @@ sub BUILD {
 
     $self->{throw_on_error} ||= 1;
 
-    $self->{debug} //=
-        $ENV{PERL_PEGEX_DEBUG} //
-        $Pegex::Parser::Debug // 0;
+    $self->{debug_indent} //=
+        $ENV{PERL_PEGEX_DEBUG_INDENT} //
+        $Pegex::Parser::DebugIndent // 1;
+    $self->{debug_indent} = 1 if (
+        not length $self->{debug_indent}
+        or $self->{debug_indent} =~ tr/0-9//c
+        or $self->{debug_indent} < 0
+    );
+    $self->{debug_color} //=
+        $ENV{PERL_PEGEX_DEBUG_COLOR} //
+        $Pegex::Parser::DebugColor // 0;
     $self->{recursion_limit} //=
         $ENV{PERL_PEGEX_RECURSION_LIMIT} //
         $Pegex::Parser::RecursionLimit // 0;
@@ -273,7 +283,24 @@ sub trace {
     my $indent = ($action =~ /^try_/) ? 1 : 0;
     $self->{indent} ||= 0;
     $self->{indent}-- unless $indent;
-    print STDERR ' ' x $self->{indent};
+
+    my $color_config = $self->{debug_color};
+    if ($color_config) {
+        my ($when, $got, $not) = map { $_ // '' } split / *, */, $color_config;
+        $got = 'green' unless length $got;
+        $not = 'bright_red' unless length $not;
+        $when = 'auto' if (not length $when or $when eq 1);
+        $_ = [split ' ', $_] for ($got, $not);
+        if ($when eq 'auto' and -t STDERR or $when eq 'always') {
+            my $ansicolor = eval { require Term::ANSIColor };
+            if ($ansicolor and $action =~ m/^(got|not)/) {
+                my $color = { got => $got, not => $not }->{ $1 };
+                $action = Term::ANSIColor::colored($color, $action);
+            }
+        }
+    }
+
+    print STDERR ' ' x ($self->{indent} * $self->{debug_indent});
     $self->{indent}++ if $indent;
     my $snippet = substr(${$self->{buffer}}, $self->{position});
     $snippet = substr($snippet, 0, 30) . "..."
